﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Extensions.Collections;       //RCB4を制御するとき使うので、設定しておく
using Rcb4;                         //


namespace RCB4refarence
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();


        }


        /// <summary>
        /// 距離センサデータを取得する
        /// </summary>
        /// <returns></returns>
        private int GetRangeADData()
        {
            
            int getSensorNum = 10;              //平均とる回数

            long dataSum = 0;                   //データを足すためのバッファ

            ByteList cmd = new ByteList();      //コマンド用にリストを確保

            Byte[] rx = new Byte[5];            //ＡＤの受け取りは5byte

            cmd.Bytes = Rcb4.Rcb4.AdRead(1);    //AD1につながっているデータを取りに行くコマンド生成

            for (int i = 0; i < getSensorNum; i++)
            {

                if (Command.Synchronize(serialPort1, cmd.Bytes, ref rx) == false)   //コマンドを送って受信する
                {
                    //失敗した場合は、
                    MessageBox.Show("読み取りにいけませんでした");
                    textBox1.Text = " ";
                    return -1;

                }
                else                                                                //成功した場合
                {
                    byte[] ADData = { rx[2], rx[3] };           //AD値を一度byte配列に代入
                    int data = BitConverter.ToInt16(ADData, 0);        //int型に変換（正確には2byteなのでshout型）
                    //    textBox1.Text = data.ToString();

                    dataSum += data;

                }
            }

           // MessageBox.Show(getSensorNum.ToString() + "回の取得が終わりました");
            //平均値を返す
            return (int)(dataSum / getSensorNum);


        }


        private Double RangeADToConvert(int ADData)
        {
            //エクセルで出した式を用いる
            double distanceData = 0.0002 * (double)ADData - 0.0048;     
            //距離は逆数で計算されるので、元に戻す
            distanceData = 1 / distanceData;

            if (distanceData < 10 || 80 < distanceData)
            {
                distanceData = -1;
            }
            return distanceData;    
        }



        private void button1_Click(object sender, EventArgs e)
        {

            int rangeData = GetRangeADData();

            textBox1.Text = RangeADToConvert(rangeData).ToString();

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                serialPort1.Open();             //COMポートを開く
            }
            catch
            {
                
                MessageBox.Show("COMが開けません！！");
              //  return;                         //失敗したらエラーを出して終わる
                serialPort1.Close();

                Application.Exit();                 
            }

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            serialPort1.Close();
        }


        /// <summary>
        /// トラックバーが移動したときに呼ばれる関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            OneServoMove(1, 0,1,trackBar1.Value);

        }


        /// <summary>
        /// サーボを１つだけ動かすための関数
        /// </summary>
        /// <param name="sioNum"></param>
        /// <param name="sercoID"></param>
        /// <param name="servoPos"></param>
        private void OneServoMove(int sioNum, int servoID,int frameSize, int servoPos)
        {

            ByteList cmd = new ByteList();

            Byte[] rx = new Byte[4];

            textBox2.Text = trackBar1.Value.ToString();

            byte sendID = (byte)(servoID * 2 + (sioNum - 1));

            cmd.Bytes = Command.RunSingleServo(sendID, (byte)frameSize, servoPos);

            Command.Synchronize(serialPort1, cmd.Bytes, ref rx);      //コマンドを送る
        }


        private void PictDrawLine(PointF startPoint, float angle, float range)
        {
            Graphics g = pictureBox1.CreateGraphics();      //ピクチャーボックス内に線が引けるようにする
            
            int pointSize = 2;

            PointF endPoint = new PointF();

            PointF centerPoint = new PointF(pictureBox1.Width/2,pictureBox1.Height / 2);

            startPoint.X += centerPoint.X;      //０点（中心）に合わせる
            startPoint.Y += centerPoint.Y;

            int defaultAngle = 90;

            float angleRad = (angle - defaultAngle) / 180 * (float)Math.PI;

            endPoint.X = range * (float)Math.Cos(angleRad) + startPoint.X;
            endPoint.Y = range * (float)Math.Sin(angleRad) + startPoint.Y;

            g.DrawLine(new Pen(Color.WhiteSmoke,2), startPoint, endPoint);
            //距離の点を表示
            g.FillEllipse(Brushes.Brown, endPoint.X - (pointSize / 2), endPoint.Y - (pointSize / 2)
                            , pointSize, pointSize);
            //中心座標を表示
            g.FillEllipse(Brushes.ForestGreen, startPoint.X - (pointSize / 2), startPoint.Y - (pointSize / 2)
                  , pointSize, pointSize);

        }

        private int servoAngleConvert(double angle)
        {
            int neutral = 7500;       //7500のとき0deg

            int servoAngle = neutral + (int)((angle / 135.0) * 4000);//±135deg =>±4000 


            //サーボ最大角、最小角の設定
            if (servoAngle < 3500) servoAngle = 3500;       //3500より小さいときは3500にする
            if (servoAngle > 11500) servoAngle = 11500;     //11500より大きいときは11500にする


            return servoAngle; 
        }

        /// <summary>
        /// 周囲距離ボタンが押された時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click(object sender, EventArgs e)
        {
            //ピクチャーボックス内にグラフィックがかけるようにする
            Graphics g = pictureBox1.CreateGraphics();
            //ピクチャーボックス内を白く塗りつぶす
            g.FillRectangle(Brushes.White, 0, 0, pictureBox1.Width, pictureBox1.Height);    


            const int rangeNum = 51;              //取得する距離データの数
            const double angleWidth = 90.0;      //サーボの振る角度
            double oneAngleData =angleWidth/( rangeNum-1);      //１回におけるサーボの角度

            
            double[] rangeData = new double[rangeNum];      //距離データを保存しておく配列

            for (int i = 0; i < rangeNum; i++)
            {
                
                float angle =(float)( -(angleWidth / 2) + (i * oneAngleData));  //測定するサーボの角度指定
                int servoAngel = servoAngleConvert(angle);
                OneServoMove(1, 0, 1, servoAngel);
                System.Threading.Thread.Sleep(200);         //動き終わるまで待つ
                int rangeDataI = GetRangeADData();          //アナログデータを読む
                rangeData[i] = RangeADToConvert(rangeDataI);
           //     if (rangeData[i] < 0) continue;             //正確な距離が取れていない時は抜ける
                PictDrawLine(new PointF(0,0),angle,(float)rangeData[i]);
            }
            
            OneServoMove(1, 0, 1, servoAngleConvert(0));       //最後は0にもどる
        }
    }
}
